"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function createSubject() {
    let done;
    const observers = [];
    const sink = {
        next: emit('next'),
        error: emit('error'),
        complete: emit('complete')
    };
    const source$ = {
        subscribe,
        [Symbol.observable]() { return this; }
    };
    return { sink, source$ };
    function emit(key) {
        return function (...args) {
            if (done) {
                return;
            }
            for (const observer of observers.slice()) {
                apply(observer, key, args);
            }
            if (key === 'next') {
                return;
            }
            observers.splice(0, observers.length);
            done = { key, args };
        };
    }
    function subscribe(observerOrNext, error, complete) {
        const observer = toObserver(observerOrNext, error, complete);
        if (done) {
            const { key, args } = done;
            apply(observer, key, args);
            return { unsubscribe() { } };
        }
        observers.push(observer);
        return { unsubscribe };
        function unsubscribe() {
            const i = observers.indexOf(observer);
            if (i >= 0) {
                observers.splice(i, 1);
            }
        }
    }
}
exports.default = createSubject;
function toObserver(observerOrNext = nop, error = nop, complete = nop) {
    return typeof observerOrNext !== 'function'
        ? toObserver(observerOrNext.next.bind(observerOrNext), observerOrNext.error.bind(observerOrNext), observerOrNext.complete.bind(observerOrNext))
        : { next: observerOrNext, error, complete };
}
function apply(observer, key, args) {
    ;
    observer[key](...args);
}
function nop() { }
